@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_dyntrans.c,v 1.132 2006/10/27 13:12:20 debug Exp $
+ *  $Id: cpu_dyntrans.c,v 1.145 2007/04/10 17:26:20 debug Exp $
  *
  *  Common dyntrans routines. Included from cpu_*.c.
  */
@@ -265,10 +265,6 @@ int DYNTRANS_RUN_INSTR(struct cpu *cpu)
 			    any instruction for any ISA:  */
 			unsigned char instr[1 <<
 			    DYNTRANS_INSTR_ALIGNMENT_SHIFT];
-#ifdef DYNTRANS_X86
-			cpu->cd.x86.cursegment = X86_S_CS;
-			cpu->cd.x86.seg_override = 0;
-#endif
 			if (!cpu->memory_rw(cpu, cpu->mem, cached_pc, &instr[0],
 			    sizeof(instr), MEM_READ, CACHE_INSTRUCTION)) {
 				fatal("XXX_run_instr(): could not read "
@@ -350,7 +346,7 @@ while (cycles-- > 0)
 		}
 	} else {
 		/*  Execute multiple instructions:  */
-		int n = 0;
+		n_instrs = 0;
 		for (;;) {
 			struct DYNTRANS_IC *ic;
 
@@ -362,13 +358,18 @@ while (cycles-- > 0)
 
 			I; I; I; I; I;   I; I; I; I; I;
 
-			n += 60;
+			I; I; I; I; I;   I; I; I; I; I;
+			I; I; I; I; I;   I; I; I; I; I;
+			I; I; I; I; I;   I; I; I; I; I;
+			I; I; I; I; I;   I; I; I; I; I;
+			I; I; I; I; I;   I; I; I; I; I;
 
-			if (n + cpu->n_translated_instrs >=
-			    N_SAFE_DYNTRANS_LIMIT)
+			I; I; I; I; I;   I; I; I; I; I;
+
+			cpu->n_translated_instrs += 120;
+			if (cpu->n_translated_instrs >= N_SAFE_DYNTRANS_LIMIT)
 				break;
 		}
-		n_instrs = n;
 	}
 
 	n_instrs += cpu->n_translated_instrs;
@@ -411,12 +412,14 @@ while (cycles-- > 0)
 /*  Not yet.  TODO  */
 			if (cpu->machine->emulated_hz > 0) {
 				if (cpu->cd.mips.compare_interrupts_pending > 0)
-					cpu_interrupt(cpu, 7);
+					INTERRUPT_ASSERT(
+					    cpu->cd.mips.irq_compare);
 			} else
 #endif
 			{
 				if (diff1 > 0 && diff2 <= 0)
-					cpu_interrupt(cpu, 7);
+					INTERRUPT_ASSERT(
+					    cpu->cd.mips.irq_compare);
 			}
 		}
 	}
@@ -485,12 +488,7 @@ void DYNTRANS_FUNCTION_TRACE(struct cpu *cpu, uint64_t f, int n_args)
 	 *  than were passed in register.
 	 */
 	for (x=0; x<n_args_to_print; x++) {
-		int64_t d;
-#if defined(DYNTRANS_X86) || defined(DYNTRANS_TRANSPUTER)
-		d = 0;		/*  TODO  */
-#else
-		/*  Args in registers:  */
-		d = cpu->cd.DYNTRANS_ARCH.
+		int64_t d = cpu->cd.DYNTRANS_ARCH.
 #ifdef DYNTRANS_ALPHA
 		    r[ALPHA_A0
 #endif
@@ -502,18 +500,6 @@ void DYNTRANS_FUNCTION_TRACE(struct cpu *cpu, uint64_t f, int n_args)
 			they go downwards, ie. 22,23 and so on  */
 		    r[24
 #endif
-#ifdef DYNTRANS_AVR32
-		    r[0		/*  TODO  */
-#endif
-#ifdef DYNTRANS_HPPA
-		    r[0		/*  TODO  */
-#endif
-#ifdef DYNTRANS_I960
-		    r[0		/*  TODO  */
-#endif
-#ifdef DYNTRANS_IA64
-		    r[0		/*  TODO  */
-#endif
 #ifdef DYNTRANS_M68K
 		    d[0		/*  TODO  */
 #endif
@@ -523,9 +509,6 @@ void DYNTRANS_FUNCTION_TRACE(struct cpu *cpu, uint64_t f, int n_args)
 #ifdef DYNTRANS_PPC
 		    gpr[3
 #endif
-#ifdef DYNTRANS_RCA180X
-		    r[0		/*  TODO  */
-#endif
 #ifdef DYNTRANS_SH
 		    r[4		/*  NetBSD seems to use 4? But 2 seems
 					to be used by other code? TODO  */
@@ -534,7 +517,7 @@ void DYNTRANS_FUNCTION_TRACE(struct cpu *cpu, uint64_t f, int n_args)
 		    r[8		/*  o0..o5  */
 #endif
 		    + x];
-#endif
+
 		symbol = get_symbol_name(&cpu->machine->symbol_context, d, &ot);
 
 		if (d > -256 && d < 256)
@@ -685,9 +668,10 @@ void DYNTRANS_PC_TO_POINTERS_GENERIC(struct cpu *cpu)
 			}
 #else
 			x1 = (cached_pc >> (64-DYNTRANS_L1N)) & mask1;
-			x2 = (cached_pc >> (64-DYNTRANS_L1N-DYNTRANS_L2N)) & mask2;
-			x3 = (cached_pc >> (64-DYNTRANS_L1N-DYNTRANS_L2N-DYNTRANS_L3N))
-			    & mask3;
+			x2 = (cached_pc >> (64-DYNTRANS_L1N-DYNTRANS_L2N))
+			    & mask2;
+			x3 = (cached_pc >> (64-DYNTRANS_L1N-DYNTRANS_L2N
+			    - DYNTRANS_L3N)) & mask3;
 			l2 = cpu->cd.DYNTRANS_ARCH.l1_64[x1];
 			l3 = l2->l3[x2];
 			if (l3->host_load[x3] != NULL) {
@@ -731,7 +715,7 @@ void DYNTRANS_PC_TO_POINTERS_GENERIC(struct cpu *cpu)
 		}
 	}
 
-	if (cpu->translation_cache_cur_ofs >= DYNTRANS_CACHE_SIZE) {
+	if (cpu->translation_cache_cur_ofs >= dyntrans_cache_size) {
 #ifdef UNSTABLE_DEVEL
 		fatal("[ dyntrans: resetting the translation cache ]\n");
 #endif
@@ -758,13 +742,21 @@ void DYNTRANS_PC_TO_POINTERS_GENERIC(struct cpu *cpu)
 		physpage_ofs = ppp->next_ofs;
 	}
 
-	/*  If the offset is 0 (or ppp is NULL), then we need to create a
-	    new "default" empty translation page.  */
+	/*
+	 *  If the offset is 0, then no translation exists yet for this
+	 *  physical address. Let's create a new page, and add it first in
+	 *  the chain.
+	 */
+	if (physpage_ofs == 0) {
+		uint32_t previous_first_page_in_chain;
 
-	if (ppp == NULL) {
 		/*  fatal("CREATING page %lli (physaddr 0x%"PRIx64"), table "
 		    "index %i\n", (long long)pagenr, (uint64_t)physaddr,
 		    (int)table_index);  */
+
+		previous_first_page_in_chain = *physpage_entryp;
+
+		/*  Insert the new page first in the chain:  */
 		*physpage_entryp = physpage_ofs =
 		    cpu->translation_cache_cur_ofs;
 
@@ -773,8 +765,13 @@ void DYNTRANS_PC_TO_POINTERS_GENERIC(struct cpu *cpu)
 
 		ppp = (struct DYNTRANS_TC_PHYSPAGE *)(cpu->translation_cache
 		    + physpage_ofs);
+
+		/*  Point to the other pages in the same chain:  */
+		ppp->next_ofs = previous_first_page_in_chain;
 	}
 
+	/*  Here, ppp points to a valid physical page struct.  */
+
 #ifdef MODE32
 	if (cpu->cd.DYNTRANS_ARCH.host_load[index] != NULL)
 		cpu->cd.DYNTRANS_ARCH.phys_page[index] = ppp;
@@ -1254,6 +1251,12 @@ void DYNTRANS_INVALIDATE_TC_CODE(struct cpu *cpu, uint64_t addr, int flags)
 		physpage_entryp = &(((uint32_t *)cpu->
 		    translation_cache)[table_index]);
 		physpage_ofs = *physpage_entryp;
+
+		/*  Return immediately if there is no code translation
+		    for this page.  */
+		if (physpage_ofs == 0)
+			return;
+
 		prev_ppp = ppp = NULL;
 
 		/*  Traverse the physical page chain:  */
@@ -1271,8 +1274,10 @@ void DYNTRANS_INVALIDATE_TC_CODE(struct cpu *cpu, uint64_t addr, int flags)
 			physpage_ofs = ppp->next_ofs;
 		}
 
+		/*  If there is no translation, there is no need to go
+		    on and try to remove it from the vph_tlb_entry array:  */
 		if (physpage_ofs == 0)
-			ppp = NULL;
+			return;
 
 #if 0
 		/*
