@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_m68k.c,v 1.14 2006/09/19 10:50:08 debug Exp $
+ *  $Id: cpu_m68k.c,v 1.18 2007/03/26 02:16:09 debug Exp $
  *
  *  Motorola 68K CPU emulation.
  */
@@ -193,39 +193,6 @@ void m68k_cpu_tlbdump(struct machine *m, int x, int rawflag)
 }
 
 
-/*
- *  m68k_cpu_gdb_stub():
- *
- *  Execute a "remote GDB" command. Returns a newly allocated response string
- *  on success, NULL on failure.
- */
-char *m68k_cpu_gdb_stub(struct cpu *cpu, char *cmd)
-{
-	fatal("m68k_cpu_gdb_stub(): TODO\n");
-	return NULL;
-}
-
-
-/*
- *  m68k_cpu_interrupt():
- */
-int m68k_cpu_interrupt(struct cpu *cpu, uint64_t irq_nr)
-{
-	fatal("m68k_cpu_interrupt(): TODO\n");
-	return 0;
-}
-
-
-/*
- *  m68k_cpu_interrupt_ack():
- */
-int m68k_cpu_interrupt_ack(struct cpu *cpu, uint64_t irq_nr)
-{
-	/*  fatal("m68k_cpu_interrupt_ack(): TODO\n");  */
-	return 0;
-}
-
-
 /*  Helper functions:  */
 static void print_two(unsigned char *instr, int *len)
 { debug(" %02x%02x", instr[*len], instr[*len+1]); (*len) += 2; }
@@ -276,7 +243,8 @@ int m68k_cpu_disassemble_instr(struct cpu *cpu, unsigned char *ib,
 			if (ib[1] >= 0x50 && ib[1] <= 0x57) {
 				print_two(ib, &len);
 				print_spaces(len);
-				debug("linkw\t%%%s,#%i\n", m68k_aname[ib[1] & 7],
+				debug("linkw\t%%%s,#%i\n",
+				    m68k_aname[ib[1] & 7],
 				    ((ib[2] << 8) + ib[3]));
 			} else if (ib[1] >= 0x58 && ib[1] <= 0x5f) {
 				print_spaces(len);
