@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips_coproc.c,v 1.60 2006/10/14 23:47:37 debug Exp $
+ *  $Id: cpu_mips_coproc.c,v 1.62 2007/02/03 16:18:56 debug Exp $
  *
  *  Emulation of MIPS coprocessors.
  */
@@ -810,7 +810,7 @@ hz = 100.0;
 				cpu->cd.mips.compare_interrupts_pending --;
 
 			/*  Clear the timer interrupt assertion (bit 7):  */
-			mips_cpu_interrupt_ack(cpu, 7);
+			cp->reg[COP0_CAUSE] &= ~0x8000;
 
 			if (tmp != (uint64_t)(int64_t)(int32_t)tmp)
 				fatal("WARNING: trying to write a 64-bit value"
