@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_ppc.c,v 1.64 2006/09/21 11:53:26 debug Exp $
+ *  $Id: cpu_ppc.c,v 1.68 2007/03/26 02:01:36 debug Exp $
  *
  *  PowerPC/POWER CPU emulation.
  */
@@ -37,6 +37,7 @@
 
 #include "cpu.h"
 #include "devices.h"
+#include "interrupt.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
@@ -56,6 +57,9 @@
 void ppc_pc_to_pointers(struct cpu *);
 void ppc32_pc_to_pointers(struct cpu *);
 
+void ppc_irq_interrupt_assert(struct interrupt *interrupt);
+void ppc_irq_interrupt_deassert(struct interrupt *interrupt);
+
 
 /*
  *  ppc_cpu_new():
@@ -215,6 +219,20 @@ int ppc_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		CPU_SETTINGS_ADD_REGISTER32(tmpstr, cpu->cd.ppc.sr[i]);
 	}
 
+	/*  Register the CPU as an interrupt handler:  */
+	{
+		struct interrupt template;
+		char name[150];
+		snprintf(name, sizeof(name), "%s", cpu->path);
+		memset(&template, 0, sizeof(template));
+		template.line = 0;
+		template.name = name;
+		template.extra = cpu;
+		template.interrupt_assert = ppc_irq_interrupt_assert;
+		template.interrupt_deassert = ppc_irq_interrupt_deassert;
+		interrupt_handler_register(&template);
+	}
+
 	return 1;
 }
 
@@ -569,132 +587,23 @@ void ppc_cpu_tlbdump(struct machine *m, int x, int rawflag)
 }
 
 
-static void add_response_word(struct cpu *cpu, char *r, uint64_t value,
-	size_t maxlen, int len)
+/*
+ *  ppc_irq_interrupt_assert():
+ */
+void ppc_irq_interrupt_assert(struct interrupt *interrupt)
 {
-	char *format = (len == 4)? "%08"PRIx64 : "%016"PRIx64;
-	if (len == 4)
-		value &= 0xffffffffULL;
-	if (cpu->byte_order == EMUL_LITTLE_ENDIAN) {
-		if (len == 4) {
-			value = ((value & 0xff) << 24) +
-				((value & 0xff00) << 8) +
-				((value & 0xff0000) >> 8) +
-				((value & 0xff000000) >> 24);
-		} else {
-			value = ((value & 0xff) << 56) +
-				((value & 0xff00) << 40) +
-				((value & 0xff0000) << 24) +
-				((value & 0xff000000ULL) << 8) +
-				((value & 0xff00000000ULL) >> 8) +
-				((value & 0xff0000000000ULL) >> 24) +
-				((value & 0xff000000000000ULL) >> 40) +
-				((value & 0xff00000000000000ULL) >> 56);
-		}
-	}
-	snprintf(r + strlen(r), maxlen - strlen(r), format, (uint64_t)value);
+	struct cpu *cpu = (struct cpu *) interrupt->extra;
+	cpu->cd.ppc.irq_asserted = 1;
 }
 
 
 /*
- *  ppc_cpu_gdb_stub():
- *
- *  Execute a "remote GDB" command. Returns a newly allocated response string
- *  on success, NULL on failure.
+ *  ppc_irq_interrupt_deassert():
  */
-char *ppc_cpu_gdb_stub(struct cpu *cpu, char *cmd)
+void ppc_irq_interrupt_deassert(struct interrupt *interrupt)
 {
-	if (strcmp(cmd, "g") == 0) {
-		int i;
-		char *r;
-		size_t wlen = cpu->is_32bit?
-		    sizeof(uint32_t) : sizeof(uint64_t);
-		size_t len = 1 + 76 * wlen;
-		r = malloc(len);
-		if (r == NULL) {
-			fprintf(stderr, "out of memory\n");
-			exit(1);
-		}
-		r[0] = '\0';
-		for (i=0; i<128; i++)
-			add_response_word(cpu, r, i, len, wlen);
-		return r;
-	}
-
-	if (cmd[0] == 'p') {
-		int regnr = strtol(cmd + 1, NULL, 16);
-		size_t wlen = cpu->is_32bit?
-		    sizeof(uint32_t) : sizeof(uint64_t);
-		size_t len = 2 * wlen + 1;
-		char *r = malloc(len);
-		r[0] = '\0';
-		if (regnr >= 0 && regnr <= 31) {
-			add_response_word(cpu, r,
-			    cpu->cd.ppc.gpr[regnr], len, wlen);
-		} else if (regnr == 0x40) {
-			add_response_word(cpu, r, cpu->pc, len, wlen);
-		} else if (regnr == 0x42) {
-			add_response_word(cpu, r, cpu->cd.ppc.cr, len, wlen);
-		} else if (regnr == 0x43) {
-			add_response_word(cpu, r, cpu->cd.ppc.spr[SPR_LR],
-			    len, wlen);
-		} else if (regnr == 0x44) {
-			add_response_word(cpu, r, cpu->cd.ppc.spr[SPR_CTR],
-			    len, wlen);
-		} else if (regnr == 0x45) {
-			add_response_word(cpu, r, cpu->cd.ppc.spr[SPR_XER],
-			    len, wlen);
-		} else {
-			/*  Unimplemented:  */
-			add_response_word(cpu, r, 0xcc000 + regnr, len, wlen);
-		}
-		return r;
-	}
-
-	fatal("ppc_cpu_gdb_stub(): TODO\n");
-	return NULL;
-}
-
-
-/*
- *  ppc_cpu_interrupt():
- *
- *  0..31 are used as BeBox interrupt numbers, 32..47 = ISA,
- *  64 is used as a "re-assert" signal to cpu->machine->md_interrupt().
- *
- *  TODO: don't hardcode to BeBox!
- */
-int ppc_cpu_interrupt(struct cpu *cpu, uint64_t irq_nr)
-{
-	/*  fatal("ppc_cpu_interrupt(): 0x%x\n", (int)irq_nr);  */
-	if (irq_nr <= 64) {
-		if (cpu->machine->md_interrupt != NULL)
-			cpu->machine->md_interrupt(
-			    cpu->machine, cpu, irq_nr, 1);
-		else
-			fatal("ppc_cpu_interrupt(): md_interrupt == NULL\n");
-	} else {
-		/*  Assert PPC IRQ:  */
-		cpu->cd.ppc.irq_asserted = 1;
-	}
-	return 1;
-}
-
-
-/*
- *  ppc_cpu_interrupt_ack():
- */
-int ppc_cpu_interrupt_ack(struct cpu *cpu, uint64_t irq_nr)
-{
-	if (irq_nr <= 64) {
-		if (cpu->machine->md_interrupt != NULL)
-			cpu->machine->md_interrupt(cpu->machine,
-			    cpu, irq_nr, 0);
-	} else {
-		/*  De-assert PPC IRQ:  */
-		cpu->cd.ppc.irq_asserted = 0;
-	}
-	return 1;
+	struct cpu *cpu = (struct cpu *) interrupt->extra;
+	cpu->cd.ppc.irq_asserted = 0;
 }
 
 
