@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sh_instr.c,v 1.44 2006/11/02 05:43:43 debug Exp $
+ *  $Id: cpu_sh_instr.c,v 1.51 2007/03/08 19:04:09 debug Exp $
  *
  *  SH instructions.
  *
@@ -195,6 +195,84 @@ X(tst_imm_r0)
 }
 
 
+/*
+ *  xor_b_imm_r0_gbr:  mem[r0+gbr] |= imm
+ *  or_b_imm_r0_gbr:   mem[r0+gbr] ^= imm
+ *  and_b_imm_r0_gbr:  mem[r0+gbr] &= imm
+ *
+ *  arg[0] = imm
+ */
+X(xor_b_imm_r0_gbr)
+{
+	uint32_t addr = cpu->cd.sh.gbr + cpu->cd.sh.r[0];
+	uint8_t *p = (uint8_t *) cpu->cd.sh.host_store[addr >> 12];
+
+	if (p != NULL) {
+		p[addr & 0xfff] ^= ic->arg[0];
+	} else {
+		uint8_t data;
+		SYNCH_PC;
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_READ, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+		data ^= ic->arg[0];
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_WRITE, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+	}
+}
+X(or_b_imm_r0_gbr)
+{
+	uint32_t addr = cpu->cd.sh.gbr + cpu->cd.sh.r[0];
+	uint8_t *p = (uint8_t *) cpu->cd.sh.host_store[addr >> 12];
+
+	if (p != NULL) {
+		p[addr & 0xfff] |= ic->arg[0];
+	} else {
+		uint8_t data;
+		SYNCH_PC;
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_READ, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+		data |= ic->arg[0];
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_WRITE, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+	}
+}
+X(and_b_imm_r0_gbr)
+{
+	uint32_t addr = cpu->cd.sh.gbr + cpu->cd.sh.r[0];
+	uint8_t *p = (uint8_t *) cpu->cd.sh.host_store[addr >> 12];
+
+	if (p != NULL) {
+		p[addr & 0xfff] &= ic->arg[0];
+	} else {
+		uint8_t data;
+		SYNCH_PC;
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_READ, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+		data &= ic->arg[0];
+		if (!cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *)&data,
+		   sizeof(data), MEM_WRITE, CACHE_DATA)) {
+			/*  Exception.  */
+			return;
+		}
+	}
+}
+
+
 /*
  *  mov_imm_rn:  Set rn to a signed 8-bit value
  *  add_imm_rn:  Add a signed 8-bit value to Rn
@@ -207,10 +285,10 @@ X(add_imm_rn) { reg(ic->arg[1]) += (int32_t)ic->arg[0]; }
 
 
 /*
- *  mov_b_rm_predec_rn:  mov.b reg,@-Rn
- *  mov_w_rm_predec_rn:  mov.w reg,@-Rn
- *  mov_l_rm_predec_rn:  mov.l reg,@-Rn
- *  stc_l_rm_predec_rn:  mov.l reg,@-Rn, with MD status bit check
+ *  mov_b_rm_predec_rn:     mov.b reg,@-Rn
+ *  mov_w_rm_predec_rn:     mov.w reg,@-Rn
+ *  mov_l_rm_predec_rn:     mov.l reg,@-Rn
+ *  stc_l_rm_predec_rn_md:  mov.l reg,@-Rn, with MD status bit check
  *
  *  arg[0] = ptr to rm  (or other register)
  *  arg[1] = ptr to rn
@@ -284,7 +362,7 @@ X(mov_l_rm_predec_rn)
 		reg(ic->arg[1]) = addr;
 	}
 }
-X(stc_l_rm_predec_rn)
+X(stc_l_rm_predec_rn_md)
 {
 	uint32_t addr = reg(ic->arg[1]) - sizeof(uint32_t);
 	uint32_t *p = (uint32_t *) cpu->cd.sh.host_store[addr >> 12];
@@ -1954,12 +2032,16 @@ X(ldtlb)
 	cpu->cd.sh.utlb_hi[urc] = cpu->cd.sh.pteh;
 	cpu->cd.sh.utlb_lo[urc] = cpu->cd.sh.ptel;
 
-	if ((old_lo & SH4_PTEL_SZ_MASK) == SH4_PTEL_SZ_4K)
-		cpu->invalidate_translation_caches(cpu,
-		    old_hi & 0xfffff000, INVALIDATE_VADDR);
-	else
-		cpu->invalidate_translation_caches(cpu,
-		    old_hi & 0xfffff000, INVALIDATE_ALL);
+	/*  Invalidate the old mapping, if it belonged to the same ASID:  */
+	if ((old_hi & SH4_PTEH_ASID_MASK) ==
+	    (cpu->cd.sh.pteh & SH4_PTEH_ASID_MASK)) {
+		if ((old_lo & SH4_PTEL_SZ_MASK) == SH4_PTEL_SZ_4K)
+			cpu->invalidate_translation_caches(cpu,
+			    old_hi & 0xfffff000, INVALIDATE_VADDR);
+		else
+			cpu->invalidate_translation_caches(cpu,
+			    old_hi & 0xfffff000, INVALIDATE_ALL);
+	}
 }
 
 
@@ -2550,15 +2632,25 @@ X(tas_b_rn)
 
 
 /*
- *  prom_emul_dreamcast:
+ *  prom_emul:
  */
-X(prom_emul_dreamcast)
+X(prom_emul)
 {
 	uint32_t old_pc;
 	SYNCH_PC;
 	old_pc = cpu->pc;
 
-	dreamcast_emul(cpu);
+	switch (cpu->machine->machine_type) {
+	case MACHINE_DREAMCAST:
+		dreamcast_emul(cpu);
+		break;
+	case MACHINE_LANDISK:
+		sh_ipl_g_emul(cpu);
+		break;
+	default:
+		fatal("SH prom_emul: unimplemented machine type.\n");
+		exit(1);
+	}
 
 	if (!cpu->running) {
 		cpu->n_translated_instrs --;
@@ -2808,9 +2900,9 @@ X(to_be_translated)
 			/*  STC Rm_BANK, Rn  */
 			ic->f = instr(copy_privileged_register);
 			ic->arg[0] = (size_t)&cpu->cd.sh.r_bank[(lo8 >> 4) & 7];
-		} else if (iword == 0x00ff) {
-			/*  PROM emulation specifically for Dreamcast  */
-			ic->f = instr(prom_emul_dreamcast);
+		} else if (iword == SH_INVALID_INSTR) {
+			/*  PROM emulation (GXemul specific)  */
+			ic->f = instr(prom_emul);
 		} else {
 			switch (lo8) {
 			case 0x02:	/*  STC SR,Rn  */
@@ -3020,7 +3112,7 @@ X(to_be_translated)
 			ic->f = instr(shld);
 		} else if ((lo8 & 0x8f) == 0x83) {
 			/*  STC.L Rm_BANK,@-Rn  */
-			ic->f = instr(stc_l_rm_predec_rn);
+			ic->f = instr(stc_l_rm_predec_rn_md);
 			ic->arg[0] = (size_t)&cpu->cd.sh.r_bank[
 			    (lo8 >> 4) & 7];	/* m */
 		} else if ((lo8 & 0x8f) == 0x87) {
@@ -3045,7 +3137,7 @@ X(to_be_translated)
 				ic->arg[0] = (size_t)&cpu->cd.sh.mach;
 				break;
 			case 0x03:	/*  STC.L SR,@-Rn  */
-				ic->f = instr(stc_l_rm_predec_rn);
+				ic->f = instr(stc_l_rm_predec_rn_md);
 				ic->arg[0] = (size_t)&cpu->cd.sh.sr;
 				break;
 			case 0x04:	/*  ROTL Rn  */
@@ -3090,7 +3182,7 @@ X(to_be_translated)
 				ic->arg[0] = (size_t)&cpu->cd.sh.macl;
 				break;
 			case 0x13:	/*  STC.L GBR,@-Rn  */
-				ic->f = instr(stc_l_rm_predec_rn);
+				ic->f = instr(mov_l_rm_predec_rn);
 				ic->arg[0] = (size_t)&cpu->cd.sh.gbr;
 				break;
 			case 0x15:	/*  CMP/PL Rn  */
@@ -3101,7 +3193,7 @@ X(to_be_translated)
 				ic->arg[0] = (size_t)&cpu->cd.sh.macl;
 				break;
 			case 0x17:	/*  LDC.L @Rm+,GBR  */
-				ic->f = instr(mov_l_arg1_postinc_to_arg0_md);
+				ic->f = instr(mov_l_arg1_postinc_to_arg0);
 				ic->arg[0] = (size_t)&cpu->cd.sh.gbr;
 				break;
 			case 0x18:	/*  SHLL8 Rn  */
@@ -3113,6 +3205,11 @@ X(to_be_translated)
 			case 0x1b:	/*  TAS.B @Rn  */
 				ic->f = instr(tas_b_rn);
 				break;
+			case 0x1e:	/*  LDC Rm,GBR  */
+				ic->f = instr(mov_rm_rn);
+				ic->arg[0] = (size_t)&cpu->cd.sh.r[r8];	/* m */
+				ic->arg[1] = (size_t)&cpu->cd.sh.gbr;
+				break;
 			case 0x20:	/*  SHAL Rn  */
 				ic->f = instr(shll_rn);  /*  NOTE: shll  */
 				break;
@@ -3125,7 +3222,7 @@ X(to_be_translated)
 				ic->arg[1] = (size_t)&cpu->cd.sh.r[r8];	/* n */
 				break;
 			case 0x23:	/*  STC.L VBR,@-Rn  */
-				ic->f = instr(stc_l_rm_predec_rn);
+				ic->f = instr(stc_l_rm_predec_rn_md);
 				ic->arg[0] = (size_t)&cpu->cd.sh.vbr;
 				break;
 			case 0x24:	/*  ROTCL Rn  */
@@ -3167,7 +3264,7 @@ X(to_be_translated)
 				ic->arg[1] = (size_t)&cpu->cd.sh.vbr;
 				break;
 			case 0x33:	/*  STC.L SSR,@-Rn  */
-				ic->f = instr(stc_l_rm_predec_rn);
+				ic->f = instr(stc_l_rm_predec_rn_md);
 				ic->arg[0] = (size_t)&cpu->cd.sh.ssr;
 				break;
 			case 0x37:	/*  LDC.L @Rm+,SSR  */
@@ -3180,7 +3277,7 @@ X(to_be_translated)
 				ic->arg[1] = (size_t)&cpu->cd.sh.ssr;
 				break;
 			case 0x43:	/*  STC.L SPC,@-Rn  */
-				ic->f = instr(stc_l_rm_predec_rn);
+				ic->f = instr(stc_l_rm_predec_rn_md);
 				ic->arg[0] = (size_t)&cpu->cd.sh.spc;
 				break;
 			case 0x47:	/*  LDC.L @Rm+,SPC  */
@@ -3422,6 +3519,18 @@ X(to_be_translated)
 			ic->f = instr(or_imm_r0);
 			ic->arg[0] = lo8;
 			break;
+		case 0xd:	/*  AND.B #imm,@(R0,GBR)  */
+			ic->f = instr(and_b_imm_r0_gbr);
+			ic->arg[0] = lo8;
+			break;
+		case 0xe:	/*  XOR.B #imm,@(R0,GBR)  */
+			ic->f = instr(xor_b_imm_r0_gbr);
+			ic->arg[0] = lo8;
+			break;
+		case 0xf:	/*  OR.B #imm,@(R0,GBR)  */
+			ic->f = instr(or_b_imm_r0_gbr);
+			ic->arg[0] = lo8;
+			break;
 		default:fatal("Unimplemented opcode 0x%x,0x%x\n",
 			    main_opcode, r8);
 			goto bad;
