@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger_cmds.c,v 1.8 2006/10/14 02:30:12 debug Exp $
+ *  $Id: debugger_cmds.c,v 1.10 2006/12/30 13:30:56 debug Exp $
  *
  *  Debugger commands. Included from debugger.c.
  */
@@ -1407,7 +1407,7 @@ static void debugger_cmd_help(struct machine *m, char *cmd_line)
 	printf("\nIn generic assignments, x must be a register or other "
 	    "writable settings\nvariable, and expr can contain registers/"
 	    "settings, numeric values, or symbol\nnames, in combination with"
-	    " parenthesis and + - * / %% ^ | operators.\nIn case there are"
+	    " parenthesis and + - * / & %% ^ | operators.\nIn case there are"
 	    " multiple matches (i.e. a symbol that has the same name as a\n"
 	    "register), you may add a prefix character as a hint: '#' for"
 	    " registers, '@'\nfor symbols, and '$' for numeric values. Use"
