@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: bus_pci.c,v 1.72 2006/10/08 02:28:58 debug Exp $
+ *  $Id: bus_pci.c,v 1.80 2007/02/11 10:03:55 debug Exp $
  *  
  *  Generic PCI bus framework. This is not a normal "device", but is used by
  *  individual PCI controllers and devices.
@@ -48,6 +48,7 @@
 
 #define BUS_PCI_C
 
+#include "bus_isa.h"
 #include "bus_pci.h"
 #include "cpu.h"
 #include "device.h"
@@ -221,7 +222,7 @@ void bus_pci_add(struct machine *machine, struct pci_data *pci_data,
 
 	if (pci_data == NULL) {
 		fatal("bus_pci_add(): pci_data == NULL!\n");
-		exit(1);
+		abort();
 	}
 
 	/*  Find the PCI device:  */
@@ -338,45 +339,13 @@ static void allocate_device_space(struct pci_device *pd,
 }
 
 
-static void bus_pci_debug_dump__2(struct pci_device *pd)
-{
-	if (pd == NULL)
-		return;
-	bus_pci_debug_dump__2(pd->next);
-	debug("bus %3i, dev %2i, func %i: %s\n",
-	    pd->bus, pd->device, pd->function, pd->name);
-}
-
-
-/*
- *  bus_pci_debug_dump():
- *
- *  Lists the attached PCI devices (in reverse).
- */
-void bus_pci_debug_dump(void *extra)
-{
-	struct pci_data *d = (struct pci_data *) extra;
-	int iadd = DEBUG_INDENTATION;
-
-	debug("pci:\n");
-	debug_indentation(iadd);
-
-	if (d->first_device == NULL)
-		debug("no devices!\n");
-	else
-		bus_pci_debug_dump__2(d->first_device);
-
-	debug_indentation(-iadd);
-}
-
-
 /*
  *  bus_pci_init():
  *
  *  This doesn't register a device, but instead returns a pointer to a struct
  *  which should be passed to other bus_pci functions when accessing the bus.
  *
- *  irq_nr is the (optional) IRQ nr that this PCI bus interrupts at.
+ *  irq_path is the interrupt path to the PCI controller.
  *
  *  pci_portbase, pci_membase, and pci_irqbase are the port, memory, and
  *  interrupt bases for PCI devices (as found in the configuration registers).
@@ -387,10 +356,10 @@ void bus_pci_debug_dump(void *extra)
  *  isa_portbase, isa_membase, and isa_irqbase are the port, memory, and
  *  interrupt bases for legacy ISA devices.
  */
-struct pci_data *bus_pci_init(struct machine *machine, int irq_nr,
+struct pci_data *bus_pci_init(struct machine *machine, char *irq_path,
 	uint64_t pci_actual_io_offset, uint64_t pci_actual_mem_offset,
-	uint64_t pci_portbase, uint64_t pci_membase, int pci_irqbase,
-	uint64_t isa_portbase, uint64_t isa_membase, int isa_irqbase)
+	uint64_t pci_portbase, uint64_t pci_membase, char *pci_irqbase,
+	uint64_t isa_portbase, uint64_t isa_membase, char *isa_irqbase)
 {
 	struct pci_data *d;
 
@@ -400,18 +369,17 @@ struct pci_data *bus_pci_init(struct machine *machine, int irq_nr,
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct pci_data));
-	d->irq_nr                = irq_nr;
+
+	d->irq_path              = strdup(irq_path);
+	d->irq_path_isa          = strdup(isa_irqbase);
+	d->irq_path_pci          = strdup(pci_irqbase);
+
 	d->pci_actual_io_offset  = pci_actual_io_offset;
 	d->pci_actual_mem_offset = pci_actual_mem_offset;
 	d->pci_portbase          = pci_portbase;
 	d->pci_membase           = pci_membase;
-	d->pci_irqbase           = pci_irqbase;
 	d->isa_portbase          = isa_portbase;
 	d->isa_membase           = isa_membase;
-	d->isa_irqbase           = isa_irqbase;
-
-	/*  Register the bus:  */
-	machine_bus_register(machine, "pci", bus_pci_debug_dump, d);
 
 	/*  Assume that the first 64KB could be used by legacy ISA devices:  */
 	d->cur_pci_portbase = d->pci_portbase + 0x10000;
@@ -509,11 +477,21 @@ PCIINIT(ali_m1543)
 	/*  Linux uses these to detect which IRQ the IDE controller uses:  */
 	PCI_SET_DATA(0x44, 0x0000000e);
 	PCI_SET_DATA(0x58, 0x00000003);
+
+	switch (machine->machine_type) {
+	case MACHINE_CATS:
+		bus_isa_init(machine, pd->pcibus->irq_path_isa,
+		    BUS_ISA_PCKBC_FORCE_USE | BUS_ISA_PCKBC_NONPCSTYLE,
+		    0x7c000000, 0x80000000);
+		break;
+	default:fatal("ali_m1543 init: unimplemented machine type\n");
+		exit(1);
+	}
 }
 
 PCIINIT(ali_m5229)
 {
-	char tmpstr[300];
+	char tmpstr[300], irqstr[300];
 
 	PCI_SET_DATA(PCI_ID_REG,
 	    PCI_ID_CODE(PCI_VENDOR_ALI, PCI_PRODUCT_ALI_M5229));
@@ -521,11 +499,21 @@ PCIINIT(ali_m5229)
 	PCI_SET_DATA(PCI_CLASS_REG, PCI_CLASS_CODE(PCI_CLASS_MASS_STORAGE,
 	    PCI_SUBCLASS_MASS_STORAGE_IDE, 0x60) + 0xc1);
 
+	switch (machine->machine_type) {
+	case MACHINE_CATS:
+		/*  CATS ISA interrupts are at footbridge irq 10:  */
+		snprintf(irqstr, sizeof(irqstr), "%s.10.isa",
+		    pd->pcibus->irq_path);
+		break;
+	default:fatal("ali_m5229 init: unimplemented machine type\n");
+		exit(1);
+	}
+
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s.%i",
 		    (long long)(pd->pcibus->isa_portbase + 0x1f0),
-		    pd->pcibus->isa_irqbase + 14);
+		    irqstr, 14);
 		device_add(machine, tmpstr);
 	}
 
@@ -730,9 +718,9 @@ PCIINIT(i31244)
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
 		char tmpstr[150];
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s.%i",
 		    (long long)(pd->pcibus->pci_actual_io_offset + 0),
-		    pd->pcibus->pci_irqbase + 0);
+		    pd->pcibus->irq_path_pci, irq & 255);
 		device_add(machine, tmpstr);
 	}
 }
@@ -828,18 +816,18 @@ PCIINIT(piix3_ide)
 
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x1f0),
-		    pd->pcibus->isa_irqbase + 14);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx "
+		    "irq=%s.isa.%i", (long long)(pd->pcibus->isa_portbase +
+		    0x1f0), pd->pcibus->irq_path_isa, 14);
 		((struct piix_ide_extra *)pd->extra)->wdc0 =
 		    device_add(machine, tmpstr);
 	}
 
 	if (diskimage_exist(machine, 2, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 3, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x170),
-		    pd->pcibus->isa_irqbase + 15);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx "
+		    "irq=%s.isa.%i", (long long)(pd->pcibus->isa_portbase +
+		    0x170), pd->pcibus->irq_path_isa, 15);
 		((struct piix_ide_extra *)pd->extra)->wdc1 =
 		    device_add(machine, tmpstr);
 	}
@@ -872,18 +860,18 @@ PCIINIT(piix4_ide)
 
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x1f0),
-		    pd->pcibus->isa_irqbase + 14);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x1f0),
+		    pd->pcibus->irq_path_isa, 14);
 		((struct piix_ide_extra *)pd->extra)->wdc0 =
 		    device_add(machine, tmpstr);
 	}
 
 	if (diskimage_exist(machine, 2, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 3, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x170),
-		    pd->pcibus->isa_irqbase + 15);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x170),
+		    pd->pcibus->irq_path_isa, 15);
 		((struct piix_ide_extra *)pd->extra)->wdc1 =
 		    device_add(machine, tmpstr);
 	}
@@ -1012,18 +1000,18 @@ PCIINIT(vt82c586_ide)
 
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x1f0),
-		    pd->pcibus->isa_irqbase + 14);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x1f0),
+		    pd->pcibus->irq_path_isa, 14);
 		((struct vt82c586_ide_extra *)pd->extra)->wdc0 =
 		    device_add(machine, tmpstr);
 	}
 
 	if (diskimage_exist(machine, 2, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 3, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x170),
-		    pd->pcibus->isa_irqbase + 15);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x170),
+		    pd->pcibus->irq_path_isa, 15);
 		((struct vt82c586_ide_extra *)pd->extra)->wdc1 =
 		    device_add(machine, tmpstr);
 	}
@@ -1052,6 +1040,15 @@ PCIINIT(symphony_83c553)
 
 	PCI_SET_DATA(PCI_BHLC_REG,
 	    PCI_BHLC_CODE(0,0, 1 /* multi-function */, 0x40,0));
+
+	switch (machine->machine_type) {
+	case MACHINE_NETWINDER:
+		bus_isa_init(machine, pd->pcibus->irq_path_isa,
+		    0, 0x7c000000, 0x80000000);
+		break;
+	default:fatal("symphony_83c553 init: unimplemented machine type\n");
+		exit(1);
+	}
 }
 
 struct symphony_82c105_extra {
@@ -1120,18 +1117,18 @@ PCIINIT(symphony_82c105)
 
 	if (diskimage_exist(machine, 0, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 1, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x1f0),
-		    pd->pcibus->isa_irqbase + 14);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x1f0),
+		    pd->pcibus->irq_path_isa, 14);
 		((struct symphony_82c105_extra *)pd->extra)->wdc0 =
 		    device_add(machine, tmpstr);
 	}
 
 	if (diskimage_exist(machine, 2, DISKIMAGE_IDE) ||
 	    diskimage_exist(machine, 3, DISKIMAGE_IDE)) {
-		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%i",
-		    (long long)(pd->pcibus->isa_portbase + 0x170),
-		    pd->pcibus->isa_irqbase + 15);
+		snprintf(tmpstr, sizeof(tmpstr), "wdc addr=0x%llx irq=%s."
+		    "isa.%i", (long long)(pd->pcibus->isa_portbase + 0x170),
+		    pd->pcibus->irq_path_isa, 15);
 		((struct symphony_82c105_extra *)pd->extra)->wdc1 =
 		    device_add(machine, tmpstr);
 	}
@@ -1151,8 +1148,8 @@ PCIINIT(symphony_82c105)
 PCIINIT(dec21143)
 {
 	uint64_t port, memaddr;
-	int irq = 0;		/*  TODO  */
-	int pci_int_line = 0x101;
+	int pci_int_line = 0x101, irq = 0, isa = 0;
+	char irqstr[200];
 	char tmpstr[200];
 
 	PCI_SET_DATA(PCI_ID_REG, PCI_ID_CODE(PCI_VENDOR_DEC,
@@ -1176,18 +1173,14 @@ PCIINIT(dec21143)
 		irq = 8 + 7;
 		pci_int_line = 0x407;
 		break;
-	case MACHINE_ALGOR:
-		/*  TODO  */
-		irq = 8 + 7;
-		pci_int_line = 0x407;
-		break;
 	case MACHINE_PREP:
-		irq = 32 + 10;
+		irq = 10;
+		isa = 1;
 		pci_int_line = 0x20a;
 		break;
 	case MACHINE_MVMEPPC:
 		/*  TODO  */
-		irq = 32 + 10;
+		irq = 10;
 		pci_int_line = 0x40a;
 		break;
 	case MACHINE_PMPPC:
@@ -1206,9 +1199,17 @@ PCIINIT(dec21143)
 
 	allocate_device_space(pd, 0x100, 0x100, &port, &memaddr);
 
+	if (isa)
+		snprintf(irqstr, sizeof(irqstr), "%s.isa.%i",
+		    pd->pcibus->irq_path_isa, irq);
+	else
+		snprintf(irqstr, sizeof(irqstr), "%s.%i",
+		    pd->pcibus->irq_path_pci, irq);
+
 	snprintf(tmpstr, sizeof(tmpstr), "dec21143 addr=0x%llx addr2=0x%llx "
-	    "irq=%i pci_little_endian=1", (long long)port, (long long)memaddr,
-	    irq);
+	    "irq=%s pci_little_endian=1", (long long)port,
+	    (long long)memaddr, irqstr);
+
 	device_add(machine, tmpstr);
 }
 
