@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_eagle.c,v 1.10 2006/02/27 05:32:26 debug Exp $
+ *  $Id: dev_eagle.c,v 1.18 2007/02/16 19:57:56 debug Exp $
  *  
  *  Motorola MPC105 "Eagle" host bridge.
  */
@@ -34,25 +34,23 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "bus_isa.h"
 #include "bus_pci.h"
 #include "cpu.h"
-#include "devices.h"
+#include "device.h"
+#include "interrupt.h"
 #include "machine.h"
 #include "memory.h"
 #include "misc.h"
 
 
 struct eagle_data {
-	int		pciirq;
+	struct interrupt irq;
+
 	struct pci_data	*pci_data;
 };
 
 
-/*
- *  dev_eagle_access():
- *
- *  Passes accesses to ISA ports 0xcf8 and 0xcfc onto bus_pci.
- */
 DEVICE_ACCESS(eagle)
 {
 	uint64_t idata = 0, odata = 0;
@@ -62,6 +60,10 @@ DEVICE_ACCESS(eagle)
 	if (writeflag == MEM_WRITE)
 		idata = memory_readmax64(cpu, data, len|MEM_PCI_LITTLE_ENDIAN);
 
+	/*
+	 *  Pass accesses to ISA ports 0xcf8 and 0xcfc onto bus_pci_*:
+	 */
+
 	switch (relative_addr) {
 	case 0:	/*  Address:  */
 		bus_pci_decompose_1(idata, &bus, &dev, &func, &reg);
@@ -81,17 +83,14 @@ DEVICE_ACCESS(eagle)
 }
 
 
-/*
- *  dev_eagle_init():
- */
-struct pci_data *dev_eagle_init(struct machine *machine, struct memory *mem,
-	int isa_irqbase, int pciirq)
+DEVINIT(eagle)
 {
 	struct eagle_data *d;
-	int pci_irqbase = 0;	/*  TODO  */
 	uint64_t pci_io_offset, pci_mem_offset;
 	uint64_t isa_portbase = 0, isa_membase = 0;
 	uint64_t pci_portbase = 0, pci_membase = 0;
+	char pci_irq_base[300];
+	char isa_irq_base[300];
 
 	d = malloc(sizeof(struct eagle_data));
 	if (d == NULL) {
@@ -99,7 +98,9 @@ struct pci_data *dev_eagle_init(struct machine *machine, struct memory *mem,
 		exit(1);
 	}
 	memset(d, 0, sizeof(struct eagle_data));
-	d->pciirq = pciirq;
+
+	/*  The interrupt path to the CPU at which we are connected:  */
+	INTERRUPT_CONNECT(devinit->interrupt_path, d->irq);
 
 	/*
 	 *  According to http://www.beatjapan.org/mirror/www.be.com/
@@ -129,31 +130,53 @@ struct pci_data *dev_eagle_init(struct machine *machine, struct memory *mem,
 	isa_portbase   = 0x80000000ULL;
 	isa_membase    = 0xc0000000ULL;
 
+	switch (devinit->machine->machine_type) {
+	case MACHINE_BEBOX:
+		snprintf(pci_irq_base, sizeof(pci_irq_base), "%s.bebox",
+		    devinit->interrupt_path);
+		snprintf(isa_irq_base, sizeof(isa_irq_base), "%s.bebox.5",
+		    devinit->interrupt_path);
+		break;
+	default:
+		snprintf(pci_irq_base, sizeof(pci_irq_base), "%s",
+		    devinit->interrupt_path);
+		snprintf(isa_irq_base, sizeof(isa_irq_base), "%s",
+		    devinit->interrupt_path);
+	}
+
 	/*  Create a PCI bus:  */
-	d->pci_data = bus_pci_init(machine, pciirq,
+	d->pci_data = bus_pci_init(devinit->machine, devinit->interrupt_path,
 	    pci_io_offset, pci_mem_offset,
-	    pci_portbase, pci_membase, pci_irqbase,
-	    isa_portbase, isa_membase, isa_irqbase);
+	    pci_portbase, pci_membase, pci_irq_base,
+	    isa_portbase, isa_membase, isa_irq_base);
 
 	/*  Add the PCI glue for the controller itself:  */
-	bus_pci_add(machine, d->pci_data, mem, 0, 0, 0, "eagle");
+	bus_pci_add(devinit->machine, d->pci_data,
+	    devinit->machine->memory, 0, 0, 0, "eagle");
 
 	/*  ADDR and DATA configuration ports in ISA space:  */
-	memory_device_register(mem, "eagle", isa_portbase + BUS_PCI_ADDR,
-	    8, dev_eagle_access, d, DM_DEFAULT, NULL);
+	memory_device_register(devinit->machine->memory, "eagle",
+	    isa_portbase + BUS_PCI_ADDR, 8, dev_eagle_access, d,
+	    DM_DEFAULT, NULL);
 
-	switch (machine->machine_type) {
+	switch (devinit->machine->machine_type) {
 	case MACHINE_BEBOX:
-		bus_pci_add(machine, d->pci_data, mem, 0, 11, 0, "i82378zb");
+		bus_isa_init(devinit->machine, isa_irq_base,
+		    BUS_ISA_IDE0 | BUS_ISA_VGA, isa_portbase, isa_membase);
+		bus_pci_add(devinit->machine, d->pci_data,
+		    devinit->machine->memory, 0, 11, 0, "i82378zb");
 		break;
 	case MACHINE_PREP:
-		bus_pci_add(machine, d->pci_data, mem, 0, 11, 0, "ibm_isa");
+		bus_pci_add(devinit->machine, d->pci_data,
+		    devinit->machine->memory, 0, 11, 0, "ibm_isa");
 		break;
 	case MACHINE_MVMEPPC:
-		switch (machine->machine_subtype) {
+		bus_isa_init(devinit->machine, isa_irq_base,
+		    BUS_ISA_LPTBASE_3BC, isa_portbase, isa_membase);
+		switch (devinit->machine->machine_subtype) {
 		case MACHINE_MVMEPPC_1600:
-			bus_pci_add(machine, d->pci_data, mem, 0, 11, 0,
-			    "i82378zb");
+			bus_pci_add(devinit->machine, d->pci_data,
+			    devinit->machine->memory, 0, 11, 0, "i82378zb");
 			break;
 		default:fatal("unimplemented machine subtype for "
 			    "eagle/mvmeppc\n");
@@ -164,6 +187,8 @@ struct pci_data *dev_eagle_init(struct machine *machine, struct memory *mem,
 		exit(1);
 	}
 
-	return d->pci_data;
+	devinit->return_ptr = d->pci_data;
+
+	return 1;
 }
 
