@@ -2,7 +2,7 @@
 #define	CPU_MIPS_H
 
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,9 +28,10 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.h,v 1.50 2006/10/14 23:47:37 debug Exp $
+ *  $Id: cpu_mips.h,v 1.55 2007/02/07 18:58:32 debug Exp $
  */
 
+#include "interrupt.h"
 #include "misc.h"
 
 struct cpu_family;
@@ -203,7 +204,7 @@ struct r4000_cache_line {
 #define	MIPS_L2N		17
 #define	MIPS_L3N		18
 
-#define	MIPS_MAX_VPH_TLB_ENTRIES	192
+#define	MIPS_MAX_VPH_TLB_ENTRIES	128
 DYNTRANS_MISC_DECLARATIONS(mips,MIPS,uint64_t)
 DYNTRANS_MISC64_DECLARATIONS(mips,MIPS,uint8_t)
 
@@ -230,6 +231,7 @@ struct mips_cpu {
 	/*  Count/compare timer:  */
 	int		compare_register_set;
 	int		compare_interrupts_pending;
+	struct interrupt irq_compare;
 	struct timer	*timer;
 
 	int		rmw;		/*  Read-Modify-Write  */
@@ -275,6 +277,8 @@ struct mips_cpu {
 
 
 /*  cpu_mips.c:  */
+void mips_cpu_interrupt_assert(struct interrupt *interrupt);
+void mips_cpu_interrupt_deassert(struct interrupt *interrupt);
 int mips_cpu_instruction_has_delayslot(struct cpu *cpu, unsigned char *ib);
 void mips_cpu_tlbdump(struct machine *m, int x, int rawflag);
 void mips_cpu_register_match(struct machine *m, char *name, 
@@ -282,8 +286,6 @@ void mips_cpu_register_match(struct machine *m, char *name,
 void mips_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs);
 int mips_cpu_disassemble_instr(struct cpu *cpu, unsigned char *instr,
         int running, uint64_t addr);
-int mips_cpu_interrupt(struct cpu *cpu, uint64_t irq_nr);
-int mips_cpu_interrupt_ack(struct cpu *cpu, uint64_t irq_nr);
 void mips_cpu_exception(struct cpu *cpu, int exccode, int tlb, uint64_t vaddr,
         /*  uint64_t pagemask,  */  int coproc_nr, uint64_t vaddr_vpn2,
         int vaddr_asid, int x_64);
