@@ -2,7 +2,7 @@
 #define	CPU_SH_H
 
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,12 +28,13 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_sh.h,v 1.33 2006/10/27 15:51:37 debug Exp $
+ *  $Id: cpu_sh.h,v 1.39 2007/04/04 19:59:24 debug Exp $
  *
  *  Note: Many things here are SH4-specific, so it probably doesn't work
  *        for SH3 emulation.
  */
 
+#include "interrupt.h"
 #include "misc.h"
 #include "sh4_cpu.h"
 
@@ -49,10 +50,12 @@ struct sh_cpu_type_def {
 	uint32_t	prr;
 };
 
-#define	SH_CPU_TYPE_DEFS		{	    \
-	{ "SH7750", 32, 4, SH4_PVR_SH7750, 0	 }, \
-	{ "SH5",    64, 5, 0,              0	 }, \
-	{ NULL,      0, 0, 0,              0	 }  }
+#define	SH_CPU_TYPE_DEFS				{	    \
+	{ "SH7750",  32, 4, SH4_PVR_SH7750, 0	                 }, \
+	{ "SH7750R", 32, 4, SH4_PVR_SH7750, SH4_PRR_7750R	 }, \
+	{ "SH7751R", 32, 4, SH4_PVR_SH7751, SH4_PRR_7751R	 }, \
+	{ "SH5",     64, 5, 0,              0	                 }, \
+	{ NULL,       0, 0, 0,              0	                 }  }
 
 
 /*
@@ -81,6 +84,8 @@ DYNTRANS_MISC_DECLARATIONS(sh,SH,uint32_t)
 #define	SH_N_ITLB_ENTRIES	4
 #define	SH_N_UTLB_ENTRIES	64
 
+#define	SH_INVALID_INSTR	0x00fb
+
 
 struct sh_cpu {
 	struct sh_cpu_type_def cpu_type;
@@ -135,13 +140,29 @@ struct sh_cpu {
 	uint16_t	intc_ipra;	/*  Interrupt Priority Registers  */
 	uint16_t	intc_iprb;
 	uint16_t	intc_iprc;
+	uint16_t	intc_iprd;
+	uint32_t	intc_intpri00;
+	uint32_t	intc_intpri04;
+	uint32_t	intc_intpri08;
+	uint32_t	intc_intpri0c;
+	uint32_t	intc_intreq00;
+	uint32_t	intc_intreq04;
+	uint32_t	intc_intmsk00;
+	uint32_t	intc_intmsk04;
+	/*  Cached and calculated values:  */
+	uint8_t		int_prio_and_pending[0x1000 / 0x20];
 	int16_t		int_to_assert;	/*  Calculated int to assert  */
 	int		int_level;	/*  Calculated int level  */
-	uint32_t	int_pending[0x1000 / 0x20 / (sizeof(uint32_t)*8)];
 
 	/*  Timer/clock functionality:  */
 	int		pclock;
 
+	/*  DMA Controller: (4 channels)  */
+	uint32_t	dmac_sar[4];
+	uint32_t	dmac_dar[4];
+	uint32_t	dmac_tcr[4];
+	uint32_t	dmac_chcr[4];
+
 
 	/*
 	 *  Instruction translation cache and Virtual->Physical->Host
@@ -191,7 +212,13 @@ struct sh_cpu {
 #define	SH_FPSCR_FR		0x00200000	/*  Register Bank Select  */
 
 
+/*  int_prio_and_pending bits:  */
+#define	SH_INT_ASSERTED		0x10
+#define	SH_INT_PRIO_MASK	0x0f
+
 /*  cpu_sh.c:  */
+void sh_cpu_interrupt_assert(struct interrupt *interrupt);
+void sh_cpu_interrupt_deassert(struct interrupt *interrupt);
 int sh_cpu_instruction_has_delayslot(struct cpu *cpu, unsigned char *ib);
 int sh_run_instr(struct cpu *cpu);
 void sh_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
@@ -208,6 +235,7 @@ int sh_memory_rw(struct cpu *cpu, struct memory *mem, uint64_t vaddr,
 	unsigned char *data, size_t len, int writeflag, int cache_flags);
 int sh_cpu_family_init(struct cpu_family *);
 
+void sh_update_interrupt_priorities(struct cpu *cpu);
 void sh_update_sr(struct cpu *cpu, uint32_t new_sr);
 void sh_exception(struct cpu *cpu, int expevt, int intevt, uint32_t vaddr);
 
