@@ -2,7 +2,7 @@
 #define	DEVICE_H
 
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: device.h,v 1.18 2006/09/05 06:13:27 debug Exp $
+ *  $Id: device.h,v 1.20 2006/12/30 13:31:00 debug Exp $
  *
  *  Device registry.  (See device.c for more info.)
  */
@@ -46,8 +46,12 @@ struct devinit {
 	uint64_t	addr;		/*  Device base address  */
 	uint64_t	addr2;		/*  Secondary address (optional)  */
 	uint64_t	len;
+
+	/*  Old-style IRQ:  */
 	int		irq_nr;
-	int		dma_irq_nr;
+
+	char		*interrupt_path;/*  Interrupt path:  */
+
 	int		in_use;
 	int		addr_mult;
 	int		pci_little_endian;
