@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: machine.c,v 1.689 2006/10/04 11:56:40 debug Exp $
+ *  $Id: machine.c,v 1.693 2007/01/28 14:15:29 debug Exp $
  */
 
 #include <stdio.h>
@@ -45,7 +45,6 @@
 #include "diskimage.h"
 #include "emul.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 #include "net.h"
@@ -67,7 +66,7 @@ struct machine_entry *first_machine_entry = NULL;
  *
  *  Returns a reasonably initialized struct machine.
  */
-struct machine *machine_new(char *name, struct emul *emul)
+struct machine *machine_new(char *name, struct emul *emul, int id)
 {
 	struct machine *m;
 	m = malloc(sizeof(struct machine));
@@ -83,6 +82,15 @@ struct machine *machine_new(char *name, struct emul *emul)
 
 	m->name = strdup(name);
 
+	/*  Full path, e.g. "emul[0].machine[0]":  */
+	m->path = malloc(strlen(emul->path) + 20);
+	if (m->path == NULL) {
+		fprintf(stderr, "out of memory\n");
+		exit(1);
+	}
+	snprintf(m->path, strlen(emul->path) + 20, "%s.machine[%i]",
+	    emul->path, id);
+
 	/*  Sane default values:  */
 	m->serial_nr = 1;
 	m->machine_type = MACHINE_NONE;
@@ -147,6 +155,9 @@ void machine_destroy(struct machine *machine)
 	if (machine->name != NULL)
 		free(machine->name);
 
+	if (machine->path != NULL)
+		free(machine->path);
+
 	/*  Remove any remaining level-1 settings:  */
 	settings_remove_all(machine->settings);
 	settings_destroy(machine->settings);
@@ -374,58 +385,6 @@ void machine_statistics_init(struct machine *machine, char *fname)
 }
 
 
-/*
- *  machine_bus_register():
- *
- *  Registers a bus in a machine.
- */
-void machine_bus_register(struct machine *machine, char *busname,
-	void (*debug_dump)(void *), void *extra)
-{
-	struct machine_bus *tmp, *last = NULL, *new;
-
-	new = zeroed_alloc(sizeof(struct machine_bus));
-	new->name = strdup(busname);
-	new->debug_dump = debug_dump;
-	new->extra = extra;
-
-	/*  Register last in the bus list:  */
-	tmp = machine->first_bus;
-	while (tmp != NULL) {
-		last = tmp;
-		tmp = tmp->next;
-	}
-
-	if (last == NULL)
-		machine->first_bus = new;
-	else
-		last->next = new;
-
-	machine->n_busses ++;
-}
-
-
-/*
- *  machine_dump_bus_info():
- *
- *  Dumps info about registered busses.
- */
-void machine_dump_bus_info(struct machine *m)
-{
-	struct machine_bus *bus = m->first_bus;
-	int iadd = DEBUG_INDENTATION;
-
-	if (m->n_busses > 0)
-		debug("busses:\n");
-	debug_indentation(iadd);
-	while (bus != NULL) {
-		bus->debug_dump(bus->extra);
-		bus = bus->next;
-	}
-	debug_indentation(-iadd);
-}
-
-
 /*
  *  machine_dumpinfo():
  *
@@ -478,8 +437,6 @@ void machine_dumpinfo(struct machine *m)
 		debug("\n");
 	}
 
-	machine_dump_bus_info(m);
-
 	diskimage_dump_info(m);
 
 	if (m->force_netboot)
@@ -939,9 +896,6 @@ void machine_setup(struct machine *machine)
 		debug("\n");
 	}
 
-	if (verbose >= 2)
-		machine_dump_bus_info(machine);
-
 	if (!machine->stable)
 		fatal("!\n!  NOTE: This machine type is not implemented well"
 		    " enough yet to run\n!  any real-world code!"
