@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,13 +25,14 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_bebox.c,v 1.3 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_bebox.c,v 1.11 2007/02/16 19:57:56 debug Exp $
  *
  *  Experimental machine for running NetBSD/bebox (see
  *  http://www.netbsd.org/Ports/bebox/ for more info.)
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "bus_isa.h"
@@ -40,26 +41,25 @@
 #include "device.h"
 #include "devices.h"
 #include "machine.h"
-#include "machine_interrupts.h"
 #include "memory.h"
 #include "misc.h"
 
 
 MACHINE_SETUP(bebox)
 {
-	struct pci_data *pci_data;
+	char tmpstr[300];
 
 	machine->machine_name = "BeBox";
+	if (machine->emulated_hz == 0)
+		machine->emulated_hz = 33000000;
 
-	machine->md_int.bebox_data = device_add(machine, "bebox");
-	machine->isa_pic_data.native_irq = 5;
-	machine->md_interrupt = isa32_interrupt;
+	snprintf(tmpstr, sizeof(tmpstr), "bebox irq=%s.cpu[%i]",
+	    machine->path, machine->bootstrap_cpu);
+	device_add(machine, tmpstr);
 
-	pci_data = dev_eagle_init(machine, machine->memory,
-	    32 /*  isa irq base */, 0 /*  pci irq: TODO */);
-
-	bus_isa_init(machine, BUS_ISA_IDE0 | BUS_ISA_VGA,
-	    0x80000000, 0xc0000000, 32, 48);
+	snprintf(tmpstr, sizeof(tmpstr), "eagle irq=%s.cpu[%i]",
+	    machine->path, machine->bootstrap_cpu);
+	device_add(machine, tmpstr);
 
 	if (!machine->prom_emulation)
 		return;
@@ -98,7 +98,8 @@ MACHINE_SETUP(bebox)
 
 	store_32bit_word(cpu, cpu->cd.ppc.gpr[6] + 32, 0);  /*  next  */
 	store_32bit_word(cpu, cpu->cd.ppc.gpr[6] + 36, 2);  /*  clock */
-	store_32bit_word(cpu, cpu->cd.ppc.gpr[6] + 40, 100);
+	store_32bit_word(cpu, cpu->cd.ppc.gpr[6] + 40,
+	    (machine->emulated_hz / 4));
 }
 
 
