@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_iyonix.c,v 1.5 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_iyonix.c,v 1.8 2006/12/30 13:31:02 debug Exp $
  */
 
 #include <stdio.h>
@@ -53,7 +53,7 @@ MACHINE_SETUP(iyonix)
 
 	device_add(machine, "ns16550 irq=0 addr=0xfe800000 in_use=0");
 
-	bus_isa_init(machine, 0, 0x90000000ULL, 0x98000000ULL, 32, 48);
+	bus_isa_init(machine, machine->path, 0, 0x90000000ULL, 0x98000000ULL);
 
 	device_add(machine, "i80321 addr=0xffffe000");
 
