@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: net.c,v 1.6 2006/09/05 06:58:29 debug Exp $
+ *  $Id: net.c,v 1.8 2007/03/24 06:40:16 debug Exp $
  *
  *  Emulated network.
  *
@@ -101,6 +101,9 @@ struct ethernet_packet_link *net_allocate_ethernet_packet_link(
 fail:
 	fprintf(stderr, "net_allocate_ethernet_packet_link(): out of memory\n");
 	exit(1);
+
+	/*  Gets rid of a compiler warning:  */
+	return NULL;
 }
 
 
