@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2007  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: useremul.c,v 1.72 2006/09/30 05:57:07 debug Exp $
+ *  $Id: useremul.c,v 1.75 2007/03/10 13:37:06 debug Exp $
  *
  *  Userland (syscall) emulation.
  *
@@ -256,11 +256,6 @@ void useremul__netbsd_setup(struct cpu *cpu, int argc, char **host_argv)
 		debug("useremul__netbsd_setup(): SH: TODO\n");
 		break;
 
-	case ARCH_X86:
-		debug("useremul__netbsd_setup(): X86: TODO\n");
-
-		break;
-
 	default:
 		fatal("useremul__netbsd_setup(): unimplemented arch\n");
 		exit(1);
@@ -1774,16 +1769,13 @@ void useremul_init(void)
 	add_useremul("NetBSD/arm", ARCH_ARM, "SA1110",
 	    useremul__netbsd, useremul__netbsd_setup);
 
-	add_useremul("NetBSD/amd64", ARCH_X86, "AMD64",
-	    useremul__netbsd, useremul__netbsd_setup);
-
-	add_useremul("NetBSD/alpha", ARCH_ALPHA, "Alpha",
+	add_useremul("NetBSD/alpha", ARCH_ALPHA, "21364",
 	    useremul__netbsd, useremul__netbsd_setup);
 
 	add_useremul("Linux/PPC64", ARCH_PPC, "PPC970",
 	    useremul__linux, useremul__linux_setup);
 
-	add_useremul("FreeBSD/Alpha", ARCH_ALPHA, "Alpha",
+	add_useremul("FreeBSD/Alpha", ARCH_ALPHA, "21364",
 	    useremul__freebsd, useremul__freebsd_setup);
 }
 
